import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;

public class ClientTalkerThread extends Thread {
    private static int num_clients = 0;
    private int this_client_number = ++num_clients;
    private final SimpleLogger logger = new SimpleLogger("OpenOrClosedServer: ClientTalkerThread " + this_client_number);
    private Socket socket;
    private BufferedReader from_client;
    private PrintWriter to_client;
    
    public ClientTalkerThread(Socket socket) {
        this.socket = socket;
        try {
            this.from_client = new BufferedReader(new InputStreamReader(socket.getInputStream()));
            this.to_client = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);
            // true: PrintWriter is line buffered
        }
        catch (IOException e) {
            logger.log("IOException in constructor: " + e);
            e.printStackTrace();
        }
    }

    @Override
    public void run() {
        try {
            logger.log("Waiting for request from client...");
            String request = from_client.readLine();
            logger.log("Got request from client: [" + request + "]");
            String[] words = request.split(" ");
            String reply2client;
            String command;
            String name;
            if (words.length != 2) {
                reply2client = "ERROR: WRONG FORMAT";
            }
            else {
                command = words[0];
                name = words[1];
                if (command.equals("OPEN")) {
                    OpenOrClosedServer.reportOpen(name);
                    reply2client = "OK";
                }
                else if (command.equals("CLOSED")) {
                    OpenOrClosedServer.reportClosed(name);
                    reply2client = "OK";
                }
                else if (command.equals("QUERY")) {
                    reply2client = OpenOrClosedServer.queryOpen(name);
                }
                else {
                    reply2client = "ERROR: UNKNOWN COMMAND";
                }
            }
            to_client.println(reply2client);
            to_client.flush();
            to_client.close();
            from_client.close();
            socket.close();
        } catch (IOException e) {
            logger.log("IOException in run: " + e);
            e.printStackTrace();
        }
    }
} // ClientTalker
