import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;

public class OpenOrClosedServer {
    private static final int PORT = 2003;
    private static HashMap<String, Boolean> isOpen = new HashMap<String, Boolean>();
    
    public static synchronized void reportOpen(String name) {
        isOpen.put(name, true);
    }

    public static synchronized void reportClosed(String name) {
        isOpen.put(name, false);
    }

    public static synchronized String queryOpen(String name) {
        if (! isOpen.containsKey(name))
            return "Unknown";
        else if (isOpen.get(name))
            return "Open";
        else
            return "Closed";
    }

    public static void main(String[] args) {
        final SimpleLogger logger = new SimpleLogger("OpenOrClosedServer");

        try {
            ServerSocket s = new ServerSocket(PORT);
            logger.log("Server socket: " + s);
            logger.log("Server listening...");
            
            while(true) {
                logger.log("Waiting for a connection from a client...");
                // Blocks until a connection occurs:
                final Socket socket = s.accept();
                logger.log("Connection from a client accepted.");
                logger.log("The new socket: " + socket);
                final ClientTalkerThread c = new ClientTalkerThread(socket);
                c.start();
            }
        } catch (IOException e) {
            logger.log("IOException: " + e);
            e.printStackTrace();
        }
    } // main
} // OpenOrClosedServer
